@@ -1,11 +1,11 @@
 extern crate libc;
 
-use self::libc::{c_int, c_uint, uint8_t};
+use self::libc::{c_int, c_uint};
 use crate::defines::{AfError, Backend};
 use crate::error::HANDLE_ERROR;
 
 extern "C" {
-    fn af_set_backend(bknd: uint8_t) -> c_int;
+    fn af_set_backend(bknd: u8) -> c_int;
     fn af_get_backend_count(num_backends: *mut c_uint) -> c_int;
     fn af_get_available_backends(backends: *mut c_int) -> c_int;
     fn af_get_active_backend(backend: *mut c_int) -> c_int;
@@ -18,7 +18,7 @@ extern "C" {
 /// - `backend` to which to switch to
 pub fn set_backend(backend: Backend) {
     unsafe {
-        let err_val = af_set_backend(backend as uint8_t);
+        let err_val = af_set_backend(backend as u8);
         HANDLE_ERROR(AfError::from(err_val));
     }
 }
