@@ -7,6 +7,7 @@ use crate::error::HANDLE_ERROR;
 use crate::seq::Seq;
 use crate::util::{AfArray, AfIndex, DimT, HasAfEnum, MutAfArray, MutAfIndex};
 
+use std::default::Default;
 use std::marker::PhantomData;
 
 #[allow(dead_code)]
@@ -62,7 +63,7 @@ extern "C" {
 /// let values  = [2.0f32, 5.0, 6.0];
 /// let arr     = Array::new(&values, dims);
 ///
-/// let mut idxr = Indexer::new();
+/// let mut idxr = Indexer::default();
 ///
 /// // `idx` is created much before idxr, thus will
 /// // stay in scope at least as long as idxr
@@ -106,6 +107,13 @@ pub struct Indexer<'object> {
 pub trait Indexable {
     /// Set indexing object for a given dimension
     ///
+    /// `is_batch` parameter is not used in most cases as it has been provided in
+    /// ArrayFire C-API to enable GFOR construct in ArrayFire C++ API. This type
+    /// of construct/idea is not exposed in rust wrapper yet. So, the user would
+    /// just need to pass `None` to this parameter while calling this function.
+    /// Since we can't have default default values and we wanted to keep this
+    /// parameter for future use cases, we just made it an `std::Option`.
+    ///
     /// # Parameters
     ///
     /// - `idxr` is mutable reference to [Indexer](./struct.Indexer.html) object which will
@@ -145,16 +153,34 @@ where
                 idxr.get() as AfIndex,
                 &SeqInternal::from_seq(self) as *const SeqInternal,
                 dim as DimT,
-                is_batch.unwrap() as c_int,
+                match is_batch {
+                    Some(value) => value as c_int,
+                    None => false as c_int,
+                },
             );
             HANDLE_ERROR(AfError::from(err_val));
         }
     }
 }
 
+impl<'object> Default for Indexer<'object> {
+    fn default() -> Self {
+        let mut temp: i64 = 0;
+        unsafe {
+            let err_val = af_create_indexers(&mut temp as MutAfIndex);
+            HANDLE_ERROR(AfError::from(err_val));
+        }
+        Self {
+            handle: temp,
+            count: 0,
+            marker: PhantomData,
+        }
+    }
+}
+
 impl<'object> Indexer<'object> {
-    #[allow(unused_mut)]
     /// Create a new Indexer object and set the dimension specific index objects later
+    #[deprecated(since = "3.7.0", note = "Use Indexer::default() instead")]
     pub fn new() -> Self {
         let mut temp: i64 = 0;
         unsafe {
@@ -174,7 +200,7 @@ impl<'object> Indexer<'object> {
         T: Indexable + 'object,
     {
         idx.set(self, dim, is_batch);
-        self.count = self.count + 1;
+        self.count += 1;
     }
 
     /// Get number of indexing objects set
@@ -182,6 +208,11 @@ impl<'object> Indexer<'object> {
         self.count
     }
 
+    /// Check if any indexing objects are set
+    pub fn is_empty(&self) -> bool {
+        self.count == 0
+    }
+
     /// Get native(ArrayFire) resource handle
     pub fn get(&self) -> i64 {
         self.handle
@@ -506,7 +537,7 @@ where
 /// //     0.5328     0.9347     0.0535
 ///
 ///
-/// let mut idxrs = Indexer::new();
+/// let mut idxrs = Indexer::default();
 /// idxrs.set_index(&indices, 0, None); // 2nd parameter is indexing dimension
 /// idxrs.set_index(&seq4gen, 1, Some(false)); // 3rd parameter indicates batch operation
 ///
@@ -553,7 +584,7 @@ where
 ///
 /// let b    = constant(2.0 as f32, Dim4::new(&[3, 3, 1, 1]));
 ///
-/// let mut idxrs = Indexer::new();
+/// let mut idxrs = Indexer::default();
 /// idxrs.set_index(&indices, 0, None); // 2nd parameter is indexing dimension
 /// idxrs.set_index(&seq4gen, 1, Some(false)); // 3rd parameter indicates batch operation
 ///
