@@ -19,8 +19,8 @@
 //! are patch/fix updates for `arrayfire-rust` & `ArrayFire` respectively,
 //! and they don't need to match.
 //!
-//! Please go through our [tutorials](./book/index.html) book for more explanations on how to
-//! use ArrayFire to speedup your code.
+//! Please go through our [tutorials](http://arrayfire.org/arrayfire-rust/arrayfire/book/index.html)
+//! book for more explanations on how to use ArrayFire to speedup your code.
 
 #![doc(
     html_logo_url = "http://www.arrayfire.com/logos/arrayfire_logo_symbol.png",
@@ -71,6 +71,9 @@ mod dim4;
 pub use crate::error::{handle_error_general, register_error_handler, Callback, ErrorCallback};
 mod error;
 
+pub use crate::event::*;
+mod event;
+
 #[cfg(feature = "indexing")]
 pub use crate::index::*;
 #[cfg(feature = "indexing")]
@@ -96,6 +99,11 @@ pub use crate::lapack::*;
 #[cfg(feature = "lapack")]
 mod lapack;
 
+#[cfg(feature = "machine_learning")]
+pub use crate::machinelearning::*;
+#[cfg(feature = "machine_learning")]
+mod machinelearning;
+
 #[cfg(feature = "macros")]
 mod macros;
 mod num;
@@ -122,6 +130,7 @@ mod statistics;
 
 pub use crate::util::{get_size, HasAfEnum, ImplicitPromote};
 pub use crate::util::{ComplexFloating, FloatingPoint, RealFloating, RealNumber};
+pub use crate::util::{ConfidenceCCInput, DeconvInput, ReduceByKeyInput};
 pub use crate::util::{CovarianceComputable, EdgeComputable, MedianComputable, MomentsComputable};
 pub use crate::util::{GrayRGBConvertible, ImageFilterType, ImageNativeType, Scanable};
 mod util;
