@@ -1,3 +1,4 @@
+extern crate half;
 extern crate libc;
 extern crate num;
 
@@ -12,6 +13,7 @@ use crate::num::Zero;
 use std::mem;
 
 pub type AfArray = self::libc::c_longlong;
+pub type AfEvent = self::libc::c_longlong;
 pub type AfIndex = self::libc::c_longlong;
 pub type CellPtr = *const self::libc::c_void;
 pub type Complex32 = Complex<f32>;
@@ -20,6 +22,7 @@ pub type DimT = self::libc::c_longlong;
 pub type Feat = *const self::libc::c_void;
 pub type Intl = self::libc::c_longlong;
 pub type MutAfArray = *mut self::libc::c_longlong;
+pub type MutAfEvent = *mut self::libc::c_longlong;
 pub type MutAfIndex = *mut self::libc::c_longlong;
 pub type MutDimT = *mut self::libc::c_longlong;
 pub type MutDouble = *mut self::libc::c_double;
@@ -182,13 +185,15 @@ pub trait HasAfEnum {
     /// aggregation of set of values for a given input type. Aggregate type
     /// alias points to below types for given input types:
     /// - `Self` for input types: `Complex<64>`, `Complex<f32>`, `f64`, `f32`, `i64`, `u64`
-    /// - `f32` for input types: `bool`
+    /// - `u32` for input types: `bool`
     /// - `u32` for input types: `u8`
     /// - `i32` for input types: `i16`
     /// - `u32` for input types: `u16`
     /// - `i32` for input types: `i32`
     /// - `u32` for input types: `u32`
     type AggregateOutType;
+    /// This type is different for b8 input type
+    type ProductOutType;
     /// This type alias points to the output type for given input type of
     /// sobel filter operation. Sobel filter output alias points to below
     /// types for given input types:
@@ -211,6 +216,7 @@ impl HasAfEnum for Complex<f32> {
     type ComplexOutType = Self;
     type MeanOutType = Self;
     type AggregateOutType = Self;
+    type ProductOutType = Self;
     type SobelOutType = Self;
 
     fn get_af_dtype() -> DType {
@@ -226,6 +232,7 @@ impl HasAfEnum for Complex<f64> {
     type ComplexOutType = Self;
     type MeanOutType = Self;
     type AggregateOutType = Self;
+    type ProductOutType = Self;
     type SobelOutType = Self;
 
     fn get_af_dtype() -> DType {
@@ -241,6 +248,7 @@ impl HasAfEnum for f32 {
     type ComplexOutType = Complex<f32>;
     type MeanOutType = Self;
     type AggregateOutType = Self;
+    type ProductOutType = Self;
     type SobelOutType = Self;
 
     fn get_af_dtype() -> DType {
@@ -256,6 +264,7 @@ impl HasAfEnum for f64 {
     type ComplexOutType = Complex<f64>;
     type MeanOutType = Self;
     type AggregateOutType = Self;
+    type ProductOutType = Self;
     type SobelOutType = Self;
 
     fn get_af_dtype() -> DType {
@@ -271,6 +280,7 @@ impl HasAfEnum for bool {
     type ComplexOutType = Complex<f32>;
     type MeanOutType = f32;
     type AggregateOutType = u32;
+    type ProductOutType = bool;
     type SobelOutType = i32;
 
     fn get_af_dtype() -> DType {
@@ -286,6 +296,7 @@ impl HasAfEnum for u8 {
     type ComplexOutType = Complex<f32>;
     type MeanOutType = f32;
     type AggregateOutType = u32;
+    type ProductOutType = u32;
     type SobelOutType = i32;
 
     fn get_af_dtype() -> DType {
@@ -301,6 +312,7 @@ impl HasAfEnum for i16 {
     type ComplexOutType = Complex<f32>;
     type MeanOutType = f32;
     type AggregateOutType = i32;
+    type ProductOutType = i32;
     type SobelOutType = i32;
 
     fn get_af_dtype() -> DType {
@@ -316,12 +328,29 @@ impl HasAfEnum for u16 {
     type ComplexOutType = Complex<f32>;
     type MeanOutType = f32;
     type AggregateOutType = u32;
+    type ProductOutType = u32;
     type SobelOutType = i32;
 
     fn get_af_dtype() -> DType {
         DType::U16
     }
 }
+impl HasAfEnum for half::f16 {
+    type InType = Self;
+    type BaseType = Self;
+    type AbsOutType = Self;
+    type ArgOutType = Self;
+    type UnaryOutType = Self;
+    type ComplexOutType = Complex<half::f16>;
+    type MeanOutType = Self;
+    type AggregateOutType = f32;
+    type ProductOutType = f32;
+    type SobelOutType = Self;
+
+    fn get_af_dtype() -> DType {
+        DType::F16
+    }
+}
 impl HasAfEnum for i32 {
     type InType = Self;
     type BaseType = Self;
@@ -331,6 +360,7 @@ impl HasAfEnum for i32 {
     type ComplexOutType = Complex<f32>;
     type MeanOutType = f32;
     type AggregateOutType = i32;
+    type ProductOutType = i32;
     type SobelOutType = i32;
 
     fn get_af_dtype() -> DType {
@@ -346,6 +376,7 @@ impl HasAfEnum for u32 {
     type ComplexOutType = Complex<f32>;
     type MeanOutType = f32;
     type AggregateOutType = u32;
+    type ProductOutType = u32;
     type SobelOutType = i32;
 
     fn get_af_dtype() -> DType {
@@ -361,6 +392,7 @@ impl HasAfEnum for i64 {
     type ComplexOutType = Complex<f64>;
     type MeanOutType = f64;
     type AggregateOutType = Self;
+    type ProductOutType = Self;
     type SobelOutType = i64;
 
     fn get_af_dtype() -> DType {
@@ -376,6 +408,7 @@ impl HasAfEnum for u64 {
     type ComplexOutType = Complex<f64>;
     type MeanOutType = f64;
     type AggregateOutType = Self;
+    type ProductOutType = Self;
     type SobelOutType = i64;
 
     fn get_af_dtype() -> DType {
@@ -758,3 +791,25 @@ impl CovarianceComputable for u16 {}
 impl CovarianceComputable for u8 {}
 impl CovarianceComputable for u64 {}
 impl CovarianceComputable for i64 {}
+
+/// Trait qualifier for confidence connected components input
+pub trait ConfidenceCCInput: HasAfEnum {}
+
+impl ConfidenceCCInput for f32 {}
+impl ConfidenceCCInput for u32 {}
+impl ConfidenceCCInput for u16 {}
+impl ConfidenceCCInput for u8 {}
+
+/// Trait qualifier for confidence connected components input
+pub trait DeconvInput: HasAfEnum {}
+
+impl DeconvInput for f32 {}
+impl DeconvInput for i16 {}
+impl DeconvInput for u16 {}
+impl DeconvInput for u8 {}
+
+/// Trait qualifier for Reduction Key type
+pub trait ReduceByKeyInput: HasAfEnum {}
+
+impl ReduceByKeyInput for i32 {}
+impl ReduceByKeyInput for u32 {}
